@@ -35,7 +35,3 @@ max_n_fval: int, optional
 from .uniform import uniform
 from .latin_hypercube import latin_hypercube
 from .util import assign_startpoints
-
-__all__ = ['uniform',
-           'latin_hypercube',
-           'assign_startpoints']
