@@ -11,7 +11,8 @@ from .reference_points import (ReferencePoint,
                                create_references)
 from .clust_color import (assign_clusters,
                           assign_clustered_colors,
-                          assign_colors)
+                          assign_colors,
+                          delete_nan_inf)
 from .misc import (process_result_list,
                    process_offset_y,
                    process_y_limits)
@@ -24,22 +25,3 @@ from .optimizer_history import (optimizer_history,
 from .profiles import (profiles,
                        profiles_lowlevel,
                        profile_lowlevel)
-
-__all__ = ["ReferencePoint",
-           "create_references",
-           "assign_clusters",
-           "assign_clustered_colors",
-           "assign_colors",
-           "process_result_list",
-           "process_offset_y",
-           "process_y_limits",
-           "waterfall",
-           "waterfall_lowlevel",
-           "parameters",
-           "parameters_lowlevel",
-           "optimizer_history",
-           "optimizer_history_lowlevel",
-           "profiles",
-           "profiles_lowlevel",
-           "profile_lowlevel"
-           ]
